
package de.ugoe.cs.autoquest.eventcore.gui;

/**
 * <p>
 * Event type for selecting a value.
 * </p>
 * 
 * @version 1.0
 * @author Patrick Harms
 */
public class ValueSelection<T> implements IInteraction {

    /**
     * <p>
     * Id for object serialization.
     * </p>
     */
    private static final long serialVersionUID = 1L;

    /**
     * <p>
     * The selected value.
     * </p>
     */
    private T selectedValue;

    /**
     * <p>
     * Constructor. Creates a new ValueSelection.
     * </p>
     * 
     * @param selectedValue
     *            the selected value
     */
    public ValueSelection(T selectedValue) {
        this.selectedValue = selectedValue;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.harms.attef.userinteraction.Interaction#getName()
     */
    public String getName() {
        return "ValueSelection";
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "select value";
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.harms.attef.userinteraction.Interaction#startsLogicalSequence()
     */
    public boolean startsLogicalSequence() {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.harms.attef.userinteraction.Interaction#finishesLogicalSequence()
     */
    public boolean finishesLogicalSequence() {
        return false;
    }

    /**
     * <p>
     * Returns the selected value associated with this event.
     * </p>
     * 
     * @return the selectedValue
     */
    public T getSelectedValue() {
        return selectedValue;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if ((obj == null) || (!(obj instanceof ValueSelection<?>))) {
            return false;
        }

        ValueSelection<?> otherValueSelection = (ValueSelection<?>) obj;

        return ((otherValueSelection != null) && ((selectedValue == otherValueSelection.selectedValue) || ((selectedValue != null) && (selectedValue
            .equals(otherValueSelection.selectedValue)))));
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return selectedValue.hashCode();
    }

}
